package com.fs.sop.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.fs.sop.domain.QwSopTempDay;

import java.util.List;

/**
 * sop任务模板规则Service接口
 *
 * @author fs
 * @date 2025-02-06
 */
public interface IQwSopTempDayService extends IService<QwSopTempDay>{

    void saveList(List<QwSopTempDay> list);


    List<QwSopTempDay> listByTempId(String id);


    void insertBatch(List<QwSopTempDay> dayList);

    QwSopTempDay info(Long id);

    boolean saveOrUpdate(QwSopTempDay day);

    void addOrUpdateBatch(List<QwSopTempDay> days);
    void removeById(Long id);

    List<QwSopTempDay> listById(List<Long> dayIds);

    int getDayNumByIdLimitOne(String tempId);
}
